package cn.kdan.cloud.pdf.office.payment.webhook;

import cn.kdan.cloud.pdf.office.api.payment.constant.PaddleAPIConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.PaddleWebhookDTO;
import cn.kdan.cloud.pdf.office.common.constant.RedisKeyConstant;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.JsonUtils;
import cn.kdan.cloud.pdf.office.payment.error.ErrorMessage;
import cn.kdan.cloud.pdf.office.payment.service.PaddleWebhookService;
import cn.kdan.cloud.pdf.office.payment.verifier.Verifier;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.beanutils.BeanUtils;
import org.jetbrains.annotations.NotNull;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.HttpEntity;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * @author ComPDFKit-WPH 2023/1/30
 * <p>
 * PaddleWebhookMonitor
 */
@Controller
@RequestMapping("/paddle")
@RequiredArgsConstructor
@Slf4j
public class PaddleWebhookMonitor {

    private final Verifier verifiers;

    private final StringRedisTemplate redisTemplate;

    private final RedissonClient redissonClient;

    private final PaddleWebhookService paddleWebhookService;

    @SneakyThrows
    @PostMapping(value = "/webhook/subscription", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    public ResponseEntity<String> paddleWebhookMonitor(@NotNull HttpEntity<String> httpEntity, HttpServletRequest request) {
        boolean isValid = verifiers.verifyDataWithSignature(Objects.requireNonNull(httpEntity.getBody()));
        if (!isValid) {
            throw new BackendRuntimeException(ErrorMessage.PAYMENT_PADDLE_WEBHOOK_NO_IS_VALID);
        }
        Map<String,Object> map = new HashMap<>();
        BeanUtils.populate(map, request.getParameterMap());
        PaddleWebhookDTO paddleWebhookDTO = JsonUtils.jsonStringToBean(JsonUtils.getJsonString(map), PaddleWebhookDTO.class);
        log.info("接收 paddle webhook：{}", paddleWebhookDTO);
        // 今天之内是否已经收到过了当前订阅的信息
        String lockKey = paddleWebhookDTO.getAlertName().concat(paddleWebhookDTO.getCheckoutId()) + "-RedissonLock";
        // 获取 RLock 对象
        RLock rLock = redissonClient.getLock(lockKey);
        try {
            // 上锁
            boolean resultLock = rLock.tryLock(30, 10, TimeUnit.SECONDS);
            // 业务逻辑开始
            if (resultLock) {
                log.info("上锁成功");
                switch (paddleWebhookDTO.getAlertName()) {
                    case PaddleAPIConstant.SUBSCRIPTION_PAYMENT_SUCCEEDED:
                        // 每月支付成功，包括首次支付成功
                        paddleWebhookService.subscriptionPaymentSucceeded(paddleWebhookDTO);
                        break;
                    case PaddleAPIConstant.SUBSCRIPTION_CANCELLED:
                        paddleWebhookService.subscriptionCancel(paddleWebhookDTO);
                        break;
                    default:
                        throw new BackendRuntimeException(ErrorMessage.PAYMENT_PADDLE_WEBHOOK_EVENT_NO_HANDLE);
                }
            }
        } catch (InterruptedException e) {
            log.info("上锁失败");
            e.printStackTrace();
        } finally {
            if (rLock.isLocked() && rLock.isHeldByCurrentThread()) {
                rLock.unlock();
            }
        }
        // 返回200 OK响应
        return ResponseEntity.ok().build();
    }

    @SneakyThrows
    public static void main(String[] args) {
        String pSignature = "alert_id=1416402945&alert_name=subscription_payment_succeeded&balance_currency=EUR&balance_earnings=667.67&balance_fee=627.36&balance_gross=386.88&balance_tax=976.03&checkout_id=6-7ae6224c32087c3-fdf5227d24&country=CA&coupon=Coupon+4&currency=USD&custom_data=custom_data&customer_name=customer_name&earnings=241.12&email=rmertz%40example.net&event_time=2023-02-13+06%3A20%3A32&fee=0.8&initial_payment=true&instalments=0&marketing_consent=&next_bill_date=2023-03-11&next_payment_amount=next_payment_amount&order_id=4&passthrough=12312312312321&payment_method=paypal&payment_tax=0.26&plan_name=Example+String&quantity=13&receipt_url=https%3A%2F%2Fsandbox-my.paddle.com%2Freceipt%2F1%2Fdbbfdb772eb810a-373a15394e&sale_gross=259.59&status=trialing&subscription_id=8&subscription_payment_id=2&subscription_plan_id=6&unit_price=unit_price&user_id=9&p_signature=PgM10D%2BZjt5FokNidIqORB2JB5KPylFL4LiWXCF4YwDhAbl6z81pMto9bb%2FR0qpAkYtOg%2F1VVA94%2BjF6PDbTIht3R0DzNxpGeV1HRBEF8AWPRAqsnhvyvyB8TCba5zjNeP96GpqWdMn4clxMh4mlVtsqov4hpb713Gg8W37hlMBeqeB0rEZStDqTrkRnLOw9EdiIZ0U8z%2F9ujAUGDG%2Be4QKBpT%2F9gbnWM9k9b%2BNTYN4AIBAMeTYta3qGkWVVwGYZyD1mgwE2CqEU3Rjh69BXUI1puZ5Ocie7Ip%2F4L0TsVR7szpOBsZavMD6usAwQOqs8V1RtI0cl0kOIu%2FLREZUCQokG%2BNl9C5AGo13et9TmnVK7gZQyB8kSdxbgkeli6R9iYvnGOkAtosG3bdMAHXyWxojod183sIGEecklsdL4IpDS8Qhw71cTLWrEPfEq015L%2FNSXFWczrJ6ExVBbCOPiXiL0Vy9SWuEt%2BjsNiuQxe3B9yWlvNL6fKX1uIPAH9PyTv8W0%2F8fkP5OJG%2BHbHXgYLCdITq3X4adG398Gv2D8gdiHUhaThfCk9JceEVKR7TOOyPpfOVvf%2B7dnLaG1OCA%2BW7JM2kl7if9J77GmhCK3IXzeiCHohUO8hSw2tBg12C9CJJjgQ8H5Ze0sbnjCCyFChbu6t%2B22RVT%2FZjl1RmNsX1E%3D";
        PaddleWebhookDTO paddleWebhookDTO = JsonUtils.jsonStringToBean(pSignature, PaddleWebhookDTO.class);


    }
}
