package cn.kdan.cloud.pdf.office.payment.webhook;

import cn.kdan.cloud.pdf.office.api.payment.dto.PaypalWebhookMonitorDTO;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.JsonUtils;
import cn.kdan.cloud.pdf.office.payment.enums.PaypalWebhookEventEnum;
import cn.kdan.cloud.pdf.office.payment.properties.PaypalProperties;
import cn.kdan.cloud.pdf.office.payment.service.PayPalWebhookService;
import com.paypal.api.payments.Event;
import com.paypal.base.Constants;
import com.paypal.base.rest.APIContext;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author ComPDFKit-WPH 2023/1/17
 * <p>
 * paypal webhook 监听
 */
@Controller
@RequestMapping("/paypal")
@Slf4j@RequiredArgsConstructor
public class PaypalWebhookMonitor {

    private final PaypalProperties properties;

    private final PayPalWebhookService payPalWebhookService;

    private final RedissonClient redissonClient;

    /**
     *  监听
     */
    @PostMapping("/plan/monitor")
    public ResponseEntity<String> paypalSubscriptionWebhookMonitor(HttpServletRequest req, HttpServletResponse resp){
        try {
            String body = getBody(req);
            PaypalWebhookMonitorDTO paypalWebhookMonitorDTO = JsonUtils.jsonStringToBean(body, PaypalWebhookMonitorDTO.class);
            // ### Api Context
            APIContext apiContext = new APIContext(properties.getClientId(), properties.getClientSecret(), properties.getMode());
            // Set the webhookId that you received when you created this webhook.
            apiContext.addConfiguration(Constants.PAYPAL_WEBHOOK_ID, properties.getWebhookId());
//            apiContext.addConfiguration(Constants.PAYPAL_WEBHOOK_ID, "6B291115V6609704N");
            boolean result = Event.validateReceivedEvent(apiContext, getHeadersInfo(
                    req), body);
            log.info("Webhook Validated:  " + result);
            log.info(body.toString());
            if (!result){
                return ResponseEntity.ok().build();
            }
            String lockKey;
            if (StringUtils.isEmpty(paypalWebhookMonitorDTO.getResource().getBilling_agreement_id())
                    && !StringUtils.isEmpty(paypalWebhookMonitorDTO.getResource().getId())){
                lockKey = paypalWebhookMonitorDTO.getEventType().concat(paypalWebhookMonitorDTO.getResource().getId()) + "-RedissonLock";
            }else {
                lockKey = paypalWebhookMonitorDTO.getEventType().concat(paypalWebhookMonitorDTO.getResource().getBilling_agreement_id()) + "-RedissonLock";
            }
            // 获取 RLock 对象
            RLock rLock = redissonClient.getLock(lockKey);
            try {
                // 上锁
                boolean resultLock = rLock.tryLock(30, 10, TimeUnit.SECONDS);
                // 业务逻辑开始
                if (resultLock) {
                    log.info("上锁成功");
                    switch (PaypalWebhookEventEnum.getByValue(paypalWebhookMonitorDTO.getEventType())) {
                        case SALE_COMPLETED:
                            // 订阅扣款成功
                            payPalWebhookService.subscriptionPaymentSucceeded(paypalWebhookMonitorDTO);
                            break;
                        case SALE_REFUNDED:
                            // 退款回调事件
                            break;
                        case BILLING_SUBSCRIPTION_PAYMENT_FAILED:
                            // 订阅付款失败。
                            payPalWebhookService.subscriptionPayFail(paypalWebhookMonitorDTO);
                            break;
                        case BILLING_SUBSCRIPTION_CANCELLED:
                            // 订阅取消成功回调
                             payPalWebhookService.subscriptionCancel(paypalWebhookMonitorDTO);
                            break;
                        default:
                            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_EVENT_IS_CURRENTLY_NOT_SUPPORTED);
                    }
                }
            } catch (InterruptedException e) {
                log.info("上锁失败");
                e.printStackTrace();
            } finally {
                if (rLock.isLocked() && rLock.isHeldByCurrentThread()) {
                    rLock.unlock();
                }
            }
        }
        //catch (PayPalRESTException | SignatureException | NoSuchAlgorithmException | InvalidKeyException | IOException e) {
        catch (Exception e) {
        log.error("Paypal回调失败："+e.getMessage(),e);
        }
        // 返回200 OK响应
        return ResponseEntity.ok().build();
    }

    // Simple helper method to help you extract the headers from HttpServletRequest object.
    private static Map< String, String > getHeadersInfo(HttpServletRequest request) {
        Map < String, String > map = new HashMap< String, String >();
        @SuppressWarnings("rawtypes")
        Enumeration headerNames = request.getHeaderNames();
        while (headerNames.hasMoreElements()) {
            String key = (String) headerNames.nextElement();
            String value = request.getHeader(key);
            map.put(key, value);
        }
        return map;
    }

    // Simple helper method to fetch request data as a string from HttpServletRequest object.
    private static String getBody(HttpServletRequest request) throws IOException {
        String body;
        StringBuilder stringBuilder = new StringBuilder();
        BufferedReader bufferedReader = null;
        try {
            InputStream inputStream = request.getInputStream();
            if (inputStream != null) {
                bufferedReader = new BufferedReader(new InputStreamReader(inputStream));
                char[] charBuffer = new char[128];
                int bytesRead = -1;
                while ((bytesRead = bufferedReader.read(charBuffer)) > 0) {
                    stringBuilder.append(charBuffer, 0, bytesRead);
                }
            } else {
                stringBuilder.append("");
            }
        } catch (IOException ex) {
            throw ex;
        } finally {
            if (bufferedReader != null) {
                try {
                    bufferedReader.close();
                } catch (IOException ex) {
                    throw ex;
                }
            }
        }
        body = stringBuilder.toString();
        return body;
    }




}
