package cn.kdan.cloud.pdf.office.website.service.impl;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserSubscriptionInfoApi;
import cn.kdan.cloud.pdf.office.api.account.vo.UserSubscriptionInfo;
import cn.kdan.cloud.pdf.office.api.product.enums.PlatformEnum;
import cn.kdan.cloud.pdf.office.api.product.feign.PrizeApi;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.PrizeVO;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.enums.PDFReaderProPlatformEnum;
import cn.kdan.cloud.pdf.office.common.enums.PrizeTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PayTypeEnum;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.utils.RedisUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.website.service.LotteryService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.Month;
import java.util.*;

@Service
@Slf4j
@RequiredArgsConstructor
@Transactional
public class LotteryServiceImpl implements LotteryService {
    @Autowired
    private RedisUtils<String, Object> redisUtils;
    @Value("${filmageUrl:123/}")
    private String filmageUrl;
    private final PrizeApi prizeApi;
    private final ProductApi productApi;
    private final UserApi userApi;
    private final UserSubscriptionInfoApi userSubscriptionInfoApi;
    // 奖品比例：1:2:3
    private static final PrizeTypeEnum[] PRIZES = {
            PrizeTypeEnum.AI_MEMBER,
            PrizeTypeEnum.COUPON,
            PrizeTypeEnum.COUPON,
            PrizeTypeEnum.COUPON,
            PrizeTypeEnum.COUPON,
            PrizeTypeEnum.COUPON,
            PrizeTypeEnum.COUPON,
            PrizeTypeEnum.COUPON,
            PrizeTypeEnum.REDEEM_VOUCHER,
            PrizeTypeEnum.REDEEM_VOUCHER
    };

    @Override
    public Integer lottery(String userId) {
        // 从 Redis 获取当前抽奖总次数
        Integer drawCount = (Integer) redisUtils.get("BlackFiveDrawCount");

        // 如果不存在，则默认设置为0
        if (drawCount == null) {
            drawCount = 0;
        }

        // 计算奖品索引
        int prizeIndex = (drawCount < 10) ? drawCount : drawCount % 10;

        // 获取对应奖品
        PrizeTypeEnum prize = PRIZES[prizeIndex];

        // 根据不同的奖品执行不同的处理方式
        switch (prize) {
            case AI_MEMBER:
                handleAiMemberPrize(userId);
                break;
            case COUPON:
                handleCouponPrize(userId);
                break;
            case REDEEM_VOUCHER:
                handleRedeemVoucherPrize(userId);
                break;
            default:
                throw new IllegalArgumentException("Unknown prize type: " + prize);
        }

        // 增加全局抽奖次数
        redisUtils.incr("BlackFiveDrawCount", 1);
        return prize.getCode();
    }

    private void handleAiMemberPrize(String userId) {
        // 处理AI会员奖品的逻辑
        // 给用户增加ai订阅和点数
        PrizeVO prizeVO = new PrizeVO();
        prizeVO.setUserId(userId);
        prizeVO.setType(PrizeTypeEnum.AI_MEMBER.getCode());
        ProductVO product = productApi.getProductByCode("ai-subscription-month").getResult();
        prizeVO.setProductId(product.getId());
        //1是黑五2是老带新
        prizeVO.setActivityType("1");
        //会员默认已经使用
        prizeVO.setValidFlag(2);
        prizeApi.insert(prizeVO);
        UserInfoVO infoVO =  userApi.getMemberInfoById(userId).getResult();
        // 检查是否存在 isAI 为 1 的项
        Optional<UserSubscriptionInfoVO> aiSubscription = infoVO.getSubscriptionInfoList().stream()
                .filter(subscription -> StringUtils.isNotEmpty(subscription.getIsAi())&& "1".equals(subscription.getIsAi())) // 筛选出 isAI 为 1 的项
                .findFirst(); // 找到第一个符合条件的项

        if (aiSubscription.isPresent()) {
            // 如果存在，则修改 point
            UserSubscriptionInfoVO subscriptionInfo = aiSubscription.get();
            UserSubscriptionInfo update = new UserSubscriptionInfo();
            update.setId(subscriptionInfo.getId());
            update.setPoint(subscriptionInfo.getPoint()+50);
            userSubscriptionInfoApi.update(update);
        } else {
            // 其他必要的设置，例如用户ID，应用ID等
            UserSubscriptionInfo newInfo = new UserSubscriptionInfo().withId(CommonUtils.generateId())
                    //status 已注册
                    .withUserId(userId).withStatus(1).withAppId("1")
                    .withPlatform(PDFReaderProPlatformEnum.AI.getCode()).withPayType(PayTypeEnum.NO_AUTO.value()).withPoint(50).withProductId(product.getId());
            userSubscriptionInfoApi.insert(newInfo);
        }
    }

    private void handleCouponPrize(String userId) {
        // 处理优惠券奖品的逻辑
        // 记录优惠券到奖品表
        PrizeVO prizeVO = new PrizeVO();
        prizeVO.setUserId(userId);
        //全平台高级版可用
        prizeVO.setType(PrizeTypeEnum.COUPON.getCode());
        prizeVO.setCode(getUpperCaseSubstringFromUUID(14));
        prizeVO.setStartDate(new Date());
        LocalDateTime localDateTime = LocalDateTime.of(2024, Month.DECEMBER, 31, 23, 59, 59);

        // 将LocalDateTime转换为Date
        Date date = Date.from(localDateTime.atZone(java.time.ZoneId.systemDefault()).toInstant());

        prizeVO.setEndDate(date);
        //1是黑五2是老带新
        prizeVO.setActivityType("1");
        prizeVO.setUsdPrice(BigDecimal.valueOf(15));
        prizeVO.setCnyPrice(BigDecimal.valueOf(100));
        //优惠券可用
        prizeVO.setValidFlag(1);
        prizeApi.insert(prizeVO);
    }

    private void handleRedeemVoucherPrize(String userId) {
        // 处理兑换券奖品的逻辑
        PrizeVO prizeVO = new PrizeVO();
        prizeVO.setUserId(userId);
        prizeVO.setType(PrizeTypeEnum.REDEEM_VOUCHER.getCode());
        prizeVO.setCode(getFilmageCode(userApi.getById(userId).getResult().getEmail()));
        prizeVO.setStartDate(new Date());
        LocalDateTime localDateTime = LocalDateTime.of(2024, Month.DECEMBER, 31, 23, 59, 59);

        // 将LocalDateTime转换为Date
        Date date = Date.from(localDateTime.atZone(java.time.ZoneId.systemDefault()).toInstant());

        prizeVO.setEndDate(date);

        //1是黑五2是老带新
        prizeVO.setActivityType("1");
        prizeVO.setVoucher("Filmage 半年序列码");
        //优惠券可用
        prizeVO.setValidFlag(1);
        prizeApi.insert(prizeVO);
    }

    public String getFilmageCode(String email) {
        String url = filmageUrl+"api/license/generate";

        // 创建 RestTemplate 实例
        RestTemplate restTemplate = new RestTemplate();

        // 创建请求头
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);

        // 创建请求体
        String jsonBody = String.format("{\"email\": \"%s\", \"name\": \"%s\", \"for_reason\": \"%s\",\"product_code\":\"%s\"}",
                email, email, "黑五活动","com.filmage.pro-lite.mac");
        HttpEntity<String> requestEntity = new HttpEntity<>(jsonBody, headers);

        // 发送 POST 请求
        try {
            String response = restTemplate.postForObject(url, requestEntity, String.class);
            System.out.println("Response: " + response);
            // 解析 JSON 响应
            JSONObject jsonResponse = JSON.parseObject(response);
            int code = jsonResponse.getInteger("code");
            // 检查 code
            if (code != 200) {
                throw new RuntimeException("Error: Received an unexpected response code: " + code);
            }
            // 提取 license
            String license = jsonResponse.getJSONObject("data").getJSONArray("license").getString(0);
            return license;
        } catch (Exception e) {
            log.error("Error: 生成flmige序列码失败 {}",email, e);
            throw new RuntimeException("Error: 生成flmige序列码失败", e);
        }
    }
    public static String getUpperCaseSubstringFromUUID(int length) {
        // 生成 UUID
        String uuid = UUID.randomUUID().toString();

        // 获取从第 4 个字符开始的子字符串
        // 注意，Java 字符串是从索引 0 开始，因此要从索引 3 开始
        String substring = uuid.substring(3, Math.min(3 + length, uuid.length()));

        // 转换为大写
        return substring.toUpperCase();
    }


}
