package cn.kdan.cloud.pdf.office.sso.utils;

import cn.kdan.cloud.pdf.office.common.utils.JsonUtils;
import cn.kdan.cloud.pdf.office.api.account.vo.OauthClientDetails;
import cn.kdan.cloud.pdf.office.common.utils.SpringContextUtils;
import cn.kdan.cloud.pdf.office.common.vo.TokenVO;
import cn.kdan.cloud.pdf.office.sso.constant.AuthConstant;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.provider.endpoint.AuthorizationEndpoint;
import org.springframework.security.oauth2.provider.endpoint.TokenEndpoint;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.bind.support.SimpleSessionStatus;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.view.RedirectView;

import java.security.Principal;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * oauth2令牌获取
 * @author tangxiangan
 */
public class TokenUtils {
    private static final Logger logger = LoggerFactory.getLogger(TokenUtils.class);

    /**
     * 应用内部获取令牌
     *
     * @param code     授权码
     * @param clientId 客户端ID
     * @param secret   凭证
     */
    public static OAuth2AccessToken getToken(String code, String clientId, String secret) {

        Map<String, String> params = new HashMap<>();
        params.put("grant_type", "authorization_code");
        params.put("code", code);
        params.put("client_id", clientId);
        params.put("redirect_uri", AuthConstant.DEFAULT_REDIRECT_URI);
        params.put("client_secret", "secret");
        User principal = new User(clientId, secret, Collections.emptySet());
        UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(principal, null, Collections.emptySet());
        try {
            TokenEndpoint tokenEndpoint = SpringContextUtils.getBean(TokenEndpoint.class);
            ResponseEntity<OAuth2AccessToken> responseEntity = tokenEndpoint.postAccessToken(authenticationToken, params);
            return responseEntity.getBody();
        } catch (HttpRequestMethodNotSupportedException e) {
            logger.error(e.getMessage(), e);

            return null;
        }
    }

    /**
     * 应用内部获取授权码（用户验证完账号名和密码之后，访问获取授权码的端口authorizationEndpoint，authorize获取授权码，然后截取返回的授权码）
     *
     * @param clientId  客户端ID
     * @param principal 认证信息
     * @return 授权码
     */
    public static String getAuthorizeCode(String clientId, Principal principal) {
        Map<String, String> params = new HashMap<>();
        params.put("response_type", "code");
        params.put("client_id", clientId);
        params.put("redirect_uri", AuthConstant.DEFAULT_REDIRECT_URI);
        SessionStatus status = new SimpleSessionStatus();

        AuthorizationEndpoint authorizationEndpoint = SpringContextUtils.getBean(AuthorizationEndpoint.class);
        ModelAndView authorizeMV = authorizationEndpoint.authorize(new HashMap<>(), params, status, principal);
        RedirectView view = (RedirectView) authorizeMV.getView();
        if (view != null && StringUtils.isNotEmpty(view.getUrl())) {
            return view.getUrl().substring(view.getUrl().indexOf("code=") + 5);
        } else {
            return null;
        }
    }

    /**
     * 远程获取oauth2令牌
     *
     * @param clientDetails oauth2客户端配置信息
     * @param code          授权码
     * @param state         声明
     */
    public static TokenVO getOAuth2Token(OauthClientDetails clientDetails, String code, String state) {

        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.set("Content-Type", "application/x-www-form-urlencoded");
        httpHeaders.set("User-Agent", "PostmanRuntime/7.29.0");

        MultiValueMap<String, String> param = new LinkedMultiValueMap<>();
        param.add("grant_type", clientDetails.getAuthorizedGrantTypes());
        param.add("code", code);
        param.add("client_id", clientDetails.getClientId());
        param.add("client_secret", clientDetails.getClientSecret());
        param.add("state", state);
        param.add("redirect_uri", clientDetails.getWebServerRedirectUri());

        RestTemplate restTemplate = SpringContextUtils.getBean(RestTemplate.class);
        ResponseEntity<String> responseEntity = restTemplate.exchange(clientDetails.getTokenUrl(), HttpMethod.POST, new HttpEntity<>(param, httpHeaders), String.class);
        if (responseEntity.getStatusCode().is2xxSuccessful()) {
            return JsonUtils.jsonStringToBean(responseEntity.getBody(), TokenVO.class);

        } else {
            logger.warn("OAuth2令牌获取失败，失败内容：{}", responseEntity);
            return null;
        }
    }

    public static TokenVO convert(OAuth2AccessToken token) {

        if (token == null) {
            return null;
        }
        TokenVO tokenPojo = new TokenVO();
        tokenPojo.setAccess_token(token.getValue());
        tokenPojo.setExpires_in(String.valueOf(token.getExpiresIn()));
        tokenPojo.setToken_type(token.getTokenType());
        tokenPojo.setRefresh_token(token.getRefreshToken().getValue());

        return tokenPojo;
    }

}
